package edu.berkeley.nlp.PCFGLA;

import java.util.List;

import edu.berkeley.nlp.syntax.Tree;
import edu.berkeley.nlp.syntax.Trees;

/**
 * This class holds a result of parser.
 */
public class ParsedResult {
	List<Tree<String>> mParsedTrees = null;
	double mLogLikelihood = 0.0;
	double mIScoreTop = 0.0;
	double mIScaleTop = 0.0;

	/**
	 * Constructor
	 *
	 */
	public ParsedResult() {}

	/**
	 * Constructor
	 *
	 * @param pts    specify the parsed trees
	 */
	public ParsedResult(List<Tree<String>> pts) {
		mParsedTrees = pts;
	}

	/**
	 * Constructor
	 *
	 * @param pts        specify the list of parsed trees
	 * @param llh        specify the logLikelihood
	 * @param iScoreTop  specify the iScore[0][length][0][0]
	 * @param iScaleTop  specify the iScale[0][length][0] if exists
	 */
	public ParsedResult(List<Tree<String>> pts, double llh,
						double iScoreTop, double iScaleTop) {
		mParsedTrees = pts;
		mLogLikelihood = llh;
		mIScoreTop = iScoreTop;
		mIScaleTop = iScaleTop;
	}

	/**
	 * Set the list of parsed trees
	 *
	 * @param pts        specify the list of parsed trees
	 */
	public void setParsedTrees(List<Tree<String>> pts) { mParsedTrees = pts; }

	/**
	 * Get the list of parsed trees
	 *
	 * @return           the list of parsed trees
	 */
	public List<Tree<String>> getParsedTrees() { return mParsedTrees; }

	/**
	 * Set logLikelihood
	 *
	 * @param llh        specify the logLikelihood
	 */
	public void setLogLikelihood(double llh) { mLogLikelihood = llh; }

	/**
	 * Get log likelihood
	 *
	 * @return           the logLikelihood
	 */
	public double getLogLikelihood() { return mLogLikelihood; }

	/**
	 * Set the top of iScore
	 *
	 * @param iScoreTop  specify the iScore[0][length][0][0]
	 */
	public void setIScoreTop(double iScoreTop) { mIScoreTop = iScoreTop; }

	/**
	 * Get the top of iScore
	 *
	 * @return           the top of iScore
	 */
	public double getIScoreTop() { return mIScoreTop; }

	/**
	 * Set the top of iScale
	 *
	 * @param iScaleTop  specify the iScale[0][length][0]
	 */
	public void setIScaleTop(double iScaleTop) { mIScaleTop = iScaleTop; }

	/**
	 * Get the top of iScale
	 *
	 * @return           the top of iScale
	 */
	public double getIScaleTop() { return mIScaleTop; }
}

